unit BedProgressBar;

interface

uses
  SysUtils, Classes,
{$IFNDEF LINUX}
  Windows, Messages, Controls, Dialogs, graphics, Forms;
{$ELSE}
  QControls, QDialogs, QGraphics, QForms, Types;
{$ENDIF}


type

  TBedProgressBar = class(TGraphicControl)
  private
    { Private declarations }
    FDoubleBuffer : tbitmap;
    FGradiantBuffer : tbitmap;
    FBGImage : TBitmap;

    FBGColor : TColor; //background color

    FMin : integer;
    FMax : integer;

    FLeftPos : integer;
    FLeftPercent : single;
    FLeftColor : TColor;

    FRightPos : integer;
    FRightPercent : single;
    FRightColor : TColor;

    //3D Look
    FBGBevel : TBevelCut;
    FLeftBevel : TBevelCut;
    FRightBevel : TBevelCut;

    //options
    FShowLeftPercent : boolean;
    FShowRightPercent : boolean;
    FRightBarIsLeft : boolean;
    FGradiant : boolean;

    FLeftFont : TFont;
    FRightFont : TFont;


    procedure ChangeFont(index : integer; f : TFont);
    procedure ChangeBevel(index : integer; b : tBevelCut);
    procedure ChangeColor(index : integer; x : tcolor);
    procedure ChangePOS(index : integer; x : integer);
    procedure ChangeOption(index : integer; b : boolean);
  protected
    { Protected declarations }
    procedure paint; override;
    procedure Resize; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Assign(Source: TBedProgressBar); reintroduce;// This takes the place of the parent's Assign method
  published
    { Published declarations }
    property Align;
    property Anchors;
    property Constraints;
    property Enabled;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;


    property BackGroundImage: TBitmap read fbgimage write fbgimage default nil;
    property BackGroundColor: TColor index 1 read fbgcolor write changecolor default clblack;
    property LeftColor: TColor index 2 read fleftColor write changecolor default clblue;
    property RightColor: TColor index 3 read frightColor write changecolor default clred;

    property Min: integer index 1 read FMin write ChangePOS default 10;
    property Max: integer index 2 read FMax write ChangePOS default 10;
    property LeftPos: integer index 3 read FLeftPOS write ChangePOS default 10;
    property RightPos: integer index 4 read FRightPOS write ChangePOS default 10;
    property LeftPercent: single read FLeftPercent ;
    property RightPercent: single read FRightPercent ;

    property BevelBG: TBevelCut index 1 read FBGBevel write ChangeBevel;
    property BevelLeft: TBevelCut index 2 read FLeftBevel write ChangeBevel;
    property BevelRight: TBevelCut index 3 read FRightBevel write ChangeBevel;

    property ShowLeftPercent : Boolean index 1 read FShowLeftPercent write ChangeOption default true;
    property ShowRightPercent : Boolean index 2 read FShowRightPercent write ChangeOption default true;
    property RightBarIsLeft : Boolean index 3 read FRightBarIsLeft write ChangeOption default true;
    property Gradiant : Boolean index 4 read FGradiant write ChangeOption default true;

    property FontLeft : TFont index 1 read FLeftFont write ChangeFont;
    property FontRight : TFont index 2 read FRightFont write ChangeFont;
  end;

procedure Register;

implementation

// Specific assign method for AMCProgressBar
procedure TBedProgressBar.Assign(Source: TBedProgressBar);
begin
    Parent := Source.Parent;
    Align := Source.Align;
    Anchors := Source.Anchors;
    Constraints := Source.Constraints;
    Enabled := Source.Enabled;
    ShowHint := Source.ShowHint;
    Visible := Source.Visible;
    BackGroundColor := Source.BackGroundColor;
    LeftColor := Source.LeftColor;
    RightColor := Source.RightColor;
    Min := Source.Min;
    Max := Source.Max;
    LeftPos := Source.LeftPos;
    RightPos := Source.RightPos;
//    LeftPercent := Source.LeftPercent; READONLY
//    RightPercent := Source.RightPercent; READONLY
    BevelBG := Source.BevelBG;
    BevelLeft := Source.BevelLeft;
    BevelRight := Source.BevelRight;
    ShowLeftPercent := Source.ShowLeftPercent;
    ShowRightPercent := Source.ShowRightPercent;
    RightBarIsLeft := Source.RightBarIsLeft;
    Gradiant := Source.Gradiant;
    FontLeft.Assign(Source.FontLeft);
    FontRight.Assign(Source.FontRight);
    Top := Source.Top;
    Left := Source.Left;
    Height := Source.Height;
    Width := Source.Width;
end;

procedure TBedProgressBar.Resize;
var tr : trect;
    i : cardinal;
    c1,c2,c3,c1lft,c1rgt,c2lft,c2rgt,c3lft,c3rgt  : cardinal;
    c1diff,c2diff,c3diff : cardinal;
begin
    if FGradiant then
    begin
        //create gradiant buffer and draw everything to there in advance
        tr.Top := 1;
        tr.bottom := height-1;
        FGradiantBuffer.width:=width;
        FGradiantBuffer.height:=height;
        c1lft := (FLeftColor and $FF0000) div $10000;
        c1rgt := (FRightColor and $FF0000) div $10000;
        c1diff := c1rgt - c1lft;
        c2lft := (FLeftColor and $00FF00) div $100;
        c2rgt := (FRightColor and $00FF00) div $100;
        c2diff := c2rgt - c2lft;
        c3lft := (FLeftColor and $0000FF);
        c3rgt := (FRightColor and $0000FF);
        c3diff := c3rgt - c3lft;
        //gradiant
        for i := 1 to width -2 do
        begin
          tr.right := i+2;
          tr.left := i;
          c1 := c1lft + ((i * c1diff) div cardinal(width));
          c2 := c2lft + ((i * c2diff) div cardinal(width));
          c3 := c3lft + ((i * c3diff) div cardinal(width));
          FGradiantBuffer.Canvas.Brush.Color := c1*$10000 + c2*$100 + c3;
          FGradiantBuffer.Canvas.FillRect(tr);
        end;
    end;
end;

constructor TBedProgressBar.Create(AOwner: Tcomponent);
begin
     inherited Create(AOwner);
     FBGColor:=clBtnFace;
     FLeftColor:=clNavy;
     FRightColor:=clRed;
     FLeftPercent:=0;
     FRightPercent:=0;
     FLeftPos:=0;
     FRightPos:=0;
     FMin:=0;
     FMax:=100;
     width:=150;
     height:=16;
     FShowLeftPercent:=true;
     FShowRightPercent:=true;
     FRightBarIsLeft:=false;
     FDoubleBuffer:=TBitmap.Create;
     FGradiantBuffer:=TBitmap.Create;
     FGradiantBuffer.width:=width;
     FGradiantBuffer.height:=height;
     FBGBevel:=bvLowered;
     FLeftBevel:=bvNone;
     FRightBevel:=bvNone;
     //this control does not need to be invalidated (paint any underlying controls)
     controlStyle:=controlStyle + [csOpaque];
     FLeftFont:=TFont.create;
     FRightFont:=TFont.create;
     FGradiant:=false;
end;

destructor TBedProgressBar.Destroy;
begin
    FGradiantBuffer.free;
    FDoubleBuffer.free;
    FLeftFont.free;
    FRightFont.free;
    inherited Destroy;
end;

procedure TBedProgressBar.paint;
var rect : Trect;
begin
    //create buffer and draw everything to there
    FDoubleBuffer.width:=width;
    FDoubleBuffer.height:=height;

    //draw background if no background bitmap set
    if FBGimage=nil then
    begin
        FDoubleBuffer.canvas.brush.color:=FBGColor;
        FDoubleBuffer.canvas.fillrect(self.GetClientRect);
    end else begin
        FDoubleBuffer.Canvas.draw(0,0,FBGimage);
    end;

    Case FBGBevel of
        bvRaised : begin
                     FDoubleBuffer.Canvas.pen.color:=clWhite;
                     FDoubleBuffer.Canvas.MoveTo(0,0); //top
                     FDoubleBuffer.canvas.LineTo(width-1,0);
                     FDoubleBuffer.Canvas.MoveTo(0,0); //left
                     FDoubleBuffer.canvas.LineTo(0,height-1);
                     FDoubleBuffer.Canvas.pen.color:=clGray;
                     FDoubleBuffer.canvas.LineTo(width-1,height-1);//bottom
                     FDoubleBuffer.canvas.LineTo(width-1,0);//right
                   end;
        bvLowered : begin
                     FDoubleBuffer.Canvas.pen.color:=clGray;
                     FDoubleBuffer.Canvas.MoveTo(0,0); //top
                     FDoubleBuffer.canvas.LineTo(width-1,0);
                     FDoubleBuffer.Canvas.MoveTo(0,0); //left
                     FDoubleBuffer.canvas.LineTo(0,height-1);
                     FDoubleBuffer.Canvas.pen.color:=clWhite;
                     FDoubleBuffer.canvas.LineTo(width-1,height-1);//bottom
                     FDoubleBuffer.canvas.LineTo(width-1,0);//right
                   end;
    end;

    //draw left bar
    rect:=self.GetClientRect;
    rect.left:=1;
    rect.top:=1;
    rect.Bottom:=rect.Bottom-1;
    rect.right:=round((width-2)*FLeftPercent / 100)+1;
    if FLeftColor<> clNone then
    begin
        FDoubleBuffer.canvas.brush.color:=FLeftColor;
        if FGradiant then
        begin
            FDoubleBuffer.canvas.CopyRect(rect,FGradiantBuffer.canvas,rect);
        end else begin
            //normal left color
            FDoubleBuffer.canvas.fillrect(rect);
        end;
    end;
    Case FLeftBevel of
        bvRaised : begin
                     FDoubleBuffer.Canvas.pen.color:=clWhite;
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //top
                     FDoubleBuffer.canvas.LineTo(rect.right,rect.top);
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //left
                     FDoubleBuffer.canvas.LineTo(rect.left,rect.bottom-1);
                     FDoubleBuffer.Canvas.pen.color:=clGray;
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.bottom-1);//bottom
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.top);//right
                   end;
        bvLowered : begin
                     FDoubleBuffer.Canvas.pen.color:=clGray;
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //top
                     FDoubleBuffer.canvas.LineTo(rect.right,rect.top);
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //left
                     FDoubleBuffer.canvas.LineTo(rect.left,rect.bottom-1);
                     FDoubleBuffer.Canvas.pen.color:=clWhite;
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.bottom-1);//bottom
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.top);//right
                   end;
    end;



    //draw right bar
    if FRightBarIsLeft then
    begin
        rect.left:=rect.right;
        rect.right:=rect.left+round(((width-2)*FRightPercent / 100));
    end else begin
        rect.right:=self.GetClientRect.right-1;
        rect.left:=round(rect.right - (FRightPercent*(width-2) / 100));
    end;
    if FRightColor<> clNone then
    begin
        FDoubleBuffer.canvas.brush.color:=FRightColor;
        FDoubleBuffer.canvas.fillrect(rect);
    end;
    Case FRightBevel of
        bvRaised : begin
                     FDoubleBuffer.Canvas.pen.color:=clWhite;
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //top
                     FDoubleBuffer.canvas.LineTo(rect.right,rect.top);
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //left
                     FDoubleBuffer.canvas.LineTo(rect.left,rect.bottom-1);
                     FDoubleBuffer.Canvas.pen.color:=clGray;
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.bottom-1);//bottom
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.top);//right
                   end;
        bvLowered : begin
                     FDoubleBuffer.Canvas.pen.color:=clGray;
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //top
                     FDoubleBuffer.canvas.LineTo(rect.right,rect.top);
                     FDoubleBuffer.Canvas.MoveTo(rect.left,rect.top); //left
                     FDoubleBuffer.canvas.LineTo(rect.left,rect.bottom-1);
                     FDoubleBuffer.Canvas.pen.color:=clWhite;
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.bottom-1);//bottom
                     FDoubleBuffer.canvas.LineTo(rect.right-1,rect.top);//right
                   end;
    end;

    //text percentages
    FDoubleBuffer.canvas.brush.style:=bsClear;
    if FShowleftpercent then
    begin
        FDoubleBuffer.canvas.font:=FLeftFont;
        FDoubleBuffer.canvas.TextOut(5,(height - canvas.TextHeight(formatfloat('#;;0',FLeftPercent)+'%')) div 2,formatfloat('#;;0',FLeftPercent)+'%');
    end;
    if FShowrightpercent then
    begin
        FDoubleBuffer.canvas.font:=FRightFont;
        FDoubleBuffer.canvas.TextOut(width-10-canvas.Textwidth(formatfloat('#;;0',FrightPercent)+'%'),(height - canvas.TextHeight(formatfloat('#;;0',FrightPercent)+'%')) div 2,formatfloat('#;;0',FrightPercent)+'%');
    end;

    //draw the finished imaged on the actual control canvas
    canvas.Draw(0,0,FDoubleBuffer);
end;

procedure TBedProgressBar.changecolor(Index: integer; x : tcolor);
begin
     {change the color, which color depends on the index}
     case index of
          1 : FBGcolor:=x;//background color
          2 : FLeftcolor:=x;//left color
          3 : FRightcolor:=x;//right color
     end; {case}
     resize;
     refresh;
end;

procedure TBedProgressBar.changeBevel(Index: integer; b : tBevelCut);
begin
     {change the color, which color depends on the index}
     case index of
          1 : FBGBevel:=b;//background color
          2 : FLeftBevel:=b;//left color
          3 : FRightBevel:=b;//right color
     end; {case}
     refresh;
end;

procedure TBedProgressBar.changeFont(Index: integer; f : TFont);
begin
     {change the color, which color depends on the index}
     case index of
          1 : FLeftFont.assign(f);//left color
          2 : FRightFont.assign(f);//right color
     end; {case}
     refresh;
end;


procedure TBedProgressBar.changeOption(Index: integer; b : Boolean);
begin
     case index of
          1 : FShowLeftPercent:=b;
          2 : FShowRightPercent:=b;
          3 : FRightBarIsLeft:=b;
          4 : FGradiant:=b;
     end; {case}
     resize;
     refresh;
end;

procedure TBedProgressBar.ChangePOS(Index: integer; x : integer);
begin
     {change the position, which position depends on the index}
     case index of
        1 : FMin := x;
        2 : FMax := x;
        3 : begin
                if (x>=FMin) or (x<=FMax) then FleftPOS := x;
                if FleftPos+FRightPos>FMax then FrightPos:=FMax-FLeftPos;
            end;
        4 : begin
                if (x>=FMin) or (x<=FMax) then FRightPOS := x;
                if FleftPos+FRightPos>FMax then FleftPos:=FMax-FRightPos;
            end ;
     end; {case}

     //update percentages
     if (FMax - FMin) >0 then
     begin
        FLeftPercent := (FLeftPos*100) / (FMax - FMin);
        FRightPercent := (FRightPos*100) / (FMax - FMin);
     end;
     refresh;
end;

procedure Register;
begin
  RegisterComponents('Bed', [TBedProgressBar]);
end;

end.
