unit BedVersion;

{Version resource retreiver component
Andrew Bednarz}

interface

uses
  Windows, SysUtils, controls, classes, dialogs, forms;

type
    TBedVersion = class(TComponent)
    private
        FCustomKey : string;
        function GetVersionField(Field: string): string;
        function GetVersionResource(index : integer) : string;
    public
    published
        property FileVersion : string index 0 read getVersionResource;
        property CompanyName : string index 1 read getVersionResource;
        property FileDescription : string index 2 read getVersionResource;
        property InternalName : string index 3 read getVersionResource;
        property LegalCopyright : string index 4 read getVersionResource;
        property LegalTrademarks : string index 5 read getVersionResource;
        property OriginalFilename : string index 6 read getVersionResource;
        property ProductName : string index 7 read getVersionResource;
        property ProductVersion : string index 8 read getVersionResource;
        property Custom : string index 9 read getVersionResource;
        property CustomKey : string read FCustomKey write FCustomKey;
    end;

procedure Register;

implementation

function TBedVersion.GetVersionField(Field: string): string;
var
    VersionInfoSize, Handle, ValueLen: cardinal;
    VersionInfo: pointer;
    Value: PChar;
    VersionInfoTranslation: ^integer;
    FileName : String;
begin
    Result := '';
    FileName := Application.ExeName;
    VersionInfoSize := GetFileVersionInfoSize(PChar(FileName),Handle);
    if VersionInfoSize > 0 then
    begin
        GetMem(VersionInfo,VersionInfoSize);
        GetFileVersionInfo(PChar(FileName),Handle,VersionInfoSize,VersionInfo);
        VerQueryValue(VersionInfo,PChar('\VarFileInfo\Translation'),pointer(VersionInfoTranslation),ValueLen);
        if VerQueryValue(VersionInfo,
                        PChar(Format('\StringFileInfo\%4.4x%4.4x\%s',
                                    [LoWord(VersionInfoTranslation^),
                                    HiWord(VersionInfoTranslation^),
                                    Field])),
                        pointer(Value),
                        ValueLen) then
            Result := string(Value);
        FreeMem(VersionInfo);
    end;
end;

procedure Register;
begin
  RegisterComponents('Bed', [TBedVersion]);
end;


function TBedVersion.GetVersionResource(index : integer): string;
begin
    case index of
        0 : result := GetVersionField('FileVersion');
        1 : result := GetVersionField('CompanyName');
        2 : result := GetVersionField('FileDescription');
        3 : result := GetVersionField('InternalName');
        4 : result := GetVersionField('LegalCopyright');
        5 : result := GetVersionField('LegalTrademarks');
        6 : result := GetVersionField('OriginalFilename');
        7 : result := GetVersionField('ProductName');
        8 : result := GetVersionField('ProductVersion');
        9 : result := GetVersionField(FCustomKey);
    end;
end;

end.
