unit Bedlog;

     {log file component}

interface

uses
  SysUtils, Classes,
{$IFNDEF LINUX}
  Windows, Messages, Controls, Dialogs, SyncObjs;
{$ELSE}
  QControls, QDialogs, SyncObjs;
{$ENDIF}

type
    TBedLogFile = class(TComponent)
    private
           FFilename : string;
           FMaxSize : integer;
           LogFile: TextFile;
           LogCriticalSection: TCriticalSection;
    public
          procedure Log(s :string);
          procedure ClearLog;
    published
          property Filename: string read FFilename write FFilename;
          property MaxSize: integer read FMaxSize write FMaxSize;
    end;

procedure Register;

implementation

procedure TBedLogFile.clearlog;
begin
  try
     assignfile(logfile,FFileName);
     rewrite(LogFile);
     closefile(logfile);
  except
    showmessage('BedLOG: Clear Error');
  end;
end;

procedure TBedLogFile.Log(S: String);
var x, LogSize,FileHandle : integer;
    sl : Tstringlist;
begin
  try
    LogCriticalSection := TCriticalSection.Create;
    LogCriticalSection.Acquire;

    AssignFile(LogFile, FFileName);
    If FileExists(ffilename) then Append(LogFile) else rewrite(LogFile);
    writeln(LogFile,FormatDateTime('dd/mm/yyyy hh:mm:ss', Now),'  ',S);
    CloseFile(LogFile);

    //check file size
    repeat
        FileHandle := FileOpen(FFileName, fmOpenRead or fmShareDenyNone);
        //LogSize:=GetFileSize(FileHandle,nil) div 1024; //We want KiloBytes
        LogSize:=FileSeek(FileHandle,0,2) div 1024;  
        FileClose(FileHandle);
        if (FMaxSize>0) and (LogSize>FMaxSize) and (LogSize>0) then
        begin
            sl:= TStringList.create;
            sl.LoadFromFile(FFileName);
            for x:=1 to (sl.Count div 5) do sl.delete(0);
            sl.SaveToFile(FFileName);
            sl.free;
        end;
    until (LogSize <= FMaxSize) or (FMaxSize=0);

    LogCriticalSection.Release;
    LogCriticalSection.Free;
  except
    on E:Exception do showmessage('BEDLOG: Log Error: '+e.message);
  end;
end;

procedure Register;
begin
  RegisterComponents('Bed', [TBedLogFile]);
end;

end.
