{Andrew Bednarz 2002}
{bed@abednarz.net}
{http://www.abednarz.net}
unit LiveJournal;
{ refer docs: http://www.livejournal.com/doc/server/ljp.csp.xml-rpc.syncitems.html }

interface

uses forms,Classes, SysUtils, Controls, Contnrs,xmlrpctypes, xmlrpcclient, dialogs, bedlog;

type

  TSyncItem = class
  public
    ItemType : string;
    ItemID : integer;
    action : string;
    time : string;
  end;

  TSyncItemsResult = class(TObject)
  public
    items : TObjectList;
    count : integer;
    total : integer;
    constructor create;
    destructor destroy; override;
  end;

  TLJEvent = class
  public
        ItemID : integer;
        EventTime : string;
        Security : string;
        Subject : string;
        Event : string;
        Mood : string;
        Music : string;
  end;

  TGetEventsResult = class
  public
    events : TObjectList;
    constructor create;
    destructor destroy; override;
  end;

  TLiveJournal = class(TObject)
  private
    FCaller: TCaller;
    FFunc : TFunction;
    FResult : TResult;
    FStruct : TStruct;
    FSyncItemsResult : TSyncItemsResult;
    FEventsResult : TGetEventsResult;
    FLogFile : TBedLogFile;

  public
    Username : string;
    Password : string;
    ClientVersion : string;
    LastSync : string;
    MaxMoodID : integer;
    Moods : array of string;
    function Login : boolean;
    function SyncItems : TSyncItemsResult;
    function GetEvents(ID : integer) : TGetEventsResult;
    function GetLastError : string;
    constructor create;
    destructor destroy;
  end;


implementation

uses math;
{ TLiveJournal }

function TLiveJournal.Login : boolean;
var x : integer;
begin
    result:=false;
    FFunc.objectmethod := 'LJ.XMLRPC.login';
    FStruct.Clear;
    FStruct.AddItem('username',username);
    FStruct.AddItem('password',password);
    FStruct.AddItem('ver','0');
    FStruct.AddItem('clientversion',clientversion);
    FStruct.AddItem('getmoods',MaxMoodID);
    FFunc.AddParam(FStruct);
    FResult := FCaller.Execute(FFunc);
    try
        if FResult.GetStruct.hasKey('moods') then
        begin
             for x:=0 to FResult.GetStruct.GetArray('moods').Count-1 do
             begin
                  MaxMoodID := max(MaxMoodID,FResult.GetStruct.GetArray('moods').getStruct(x).GetInteger('id'));
                  Setlength(Moods,MaxMoodID);
                  Moods[FResult.GetStruct.GetArray('moods').getStruct(x).GetInteger('id')-1]:=FResult.GetStruct.GetArray('moods').getStruct(x).GetString('name');
             end;
        end;
    except
        on E:exception do FLogFile.Log('Login Exception: '+e.message);
    end;
    if not FResult.IsError then result:=true else FLogFile.Log('Login Error: '+Fresult.GetErrorString);
end;


constructor TLiveJournal.create;
begin
    FFunc := TFunction.Create;
    FCaller := TCaller.Create;
    FResult := TResult.create;
    FStruct := TStruct.create;
    FSyncItemsResult :=TSyncItemsResult.create;
    FEventsResult := TGetEventsResult.create;
    FCaller.HostName := 'www.livejournal.com';
    FCaller.HostPort := 80;
    FCaller.EndPoint := '/interface/xmlrpc';
    FLogFile := TBedLogFile.Create(nil);
    FLogFile.Filename:=extractfilepath(application.exename)+'LiveJournal.log';
end;

destructor TLiveJournal.destroy;
begin
    FFunc.free;
    FCaller.free;
    FResult.free;
    FStruct.free;
    FSyncItemsResult.free;
    FEventsResult.free;
    FLogFile.free;
end;

function TLiveJournal.GetLastError: string;
begin
  result := FResult.GetErrorString;
end;

function TLiveJournal.SyncItems: TSyncItemsResult;
var x : integer;
    SyncItem : TSyncItem;
    s : string;
begin
    try
        FSyncItemsResult.items.clear;
        FSyncItemsResult.count:=0;
        FSyncItemsResult.total:=0;
        result:=FSyncItemsResult;
        FFunc.objectmethod := 'LJ.XMLRPC.syncitems';
        FStruct.Clear;
        FStruct.AddItem('username',username);
        FStruct.AddItem('password',password);
        FStruct.AddItem('ver','0');
        FStruct.AddItem('lastsync',lastsync);
        FFunc.AddParam(FStruct);
        FResult := FCaller.Execute(FFunc);
        if not FResult.IsError then
        begin
            FSyncItemsResult.count := FResult.GetStruct.GetInteger('count');
            FSyncItemsResult.total := FResult.GetStruct.GetInteger('total');
            for x:=0 to FSyncItemsResult.count-1 do
            begin
                SyncItem := TSyncItem.create; //free'ed automically by list
                s :=FResult.GetStruct.GetArray('syncitems').GetStruct(x).GetString('item');
                SyncItem.ItemType := copy(s,1,1);
                SyncItem.ItemID := strtoint(copy(s,3,length(s)-2));
                SyncItem.action := FResult.GetStruct.GetArray('syncitems').GetStruct(x).GetString('action');
                SyncItem.time := FResult.GetStruct.GetArray('syncitems').GetStruct(x).GetString('time');
                FSyncItemsResult.items.Add(SyncItem);
            end;
        end else begin
            result:=nil;
            FLogFile.Log('SyncItems Error: '+Fresult.GetErrorString);
        end;
    except
        on E:exception do FLogFile.Log('SyncItems Exception: '+e.message);
    end;
end;

function TLiveJournal.GetEvents(ID : integer): TGetEventsResult;
var x : integer;
    event : TLJEvent;
begin
    try
        FEventsResult.events.clear;
        result:=FEventsResult;
        FFunc.objectmethod := 'LJ.XMLRPC.getevents';
        FStruct.Clear;
        FStruct.AddItem('username',username);
        FStruct.AddItem('password',password);
        FStruct.AddItem('ver','0');
        FStruct.AddItem('selecttype','one');
        FStruct.AddItem('itemid',ID);
        FFunc.AddParam(FStruct);
    except
        on E:exception do FLogFile.Log('GetEvents(1) Exception: '+e.message);
    end;
    try
        FResult := FCaller.Execute(FFunc);
    except
        on E:exception do FLogFile.Log('GetEvents(2) Exception: '+e.message);
    end;
    if (not (Fresult = nil)) and (not FResult.IsError) then
    begin
        if FResult.IsStruct and FResult.GetStruct.IsArray('events') then
        begin
            for x:=0 to FResult.GetStruct.GetArray('events').Count-1 do
            begin
                try
                    event := TLJEvent.create;//free'ed automically by list
                    if FResult.GetStruct.GetArray('events').isStruct(x) then
                    begin
                        if FResult.GetStruct.GetArray('events').GetStruct(x).HasKey('itemid') and FResult.GetStruct.GetArray('events').GetStruct(x).HasKey('eventtime') and FResult.GetStruct.GetArray('events').GetStruct(x).HasKey('event') then
                        begin
                            event.ItemID := FResult.GetStruct.GetArray('events').GetStruct(x).Getinteger('itemid');
                            event.EventTime := FResult.GetStruct.GetArray('events').GetStruct(x).GetString('eventtime');
                            if FResult.GetStruct.GetArray('events').GetStruct(x).HasKey('security') then
                            begin
                                 event.Security := FResult.GetStruct.GetArray('events').GetStruct(x).GetString('security');
                                 if event.Security = 'usemask' then event.Security:='Friends';
                                 if event.Security = 'private' then event.Security:='Private';
                            end else event.Security:='Public';
                            if FResult.GetStruct.GetArray('events').GetStruct(x).HasKey('subject') then event.Subject := FResult.GetStruct.GetArray('events').GetStruct(x).GetString('subject');
                            event.Event := FResult.GetStruct.GetArray('events').GetStruct(x).GetString('event');
                            if FResult.GetStruct.GetArray('events').GetStruct(x).HasKey('props') then
                            begin
                                //moodid
                                if FResult.GetStruct.GetArray('events').GetStruct(x).getStruct('props').hasKey('current_moodid') then
                                begin
                                   event.mood :=Moods[FResult.GetStruct.GetArray('events').GetStruct(x).getStruct('props').getInteger('current_moodid')-1];
                                end;
                                //other mood
                                if FResult.GetStruct.GetArray('events').GetStruct(x).getStruct('props').hasKey('current_mood') then
                                   event.mood :=FResult.GetStruct.GetArray('events').GetStruct(x).getStruct('props').getString('current_mood');
                                //music
                                if FResult.GetStruct.GetArray('events').GetStruct(x).getStruct('props').hasKey('current_music') then
                                  event.music :=FResult.GetStruct.GetArray('events').GetStruct(x).getStruct('props').getString('current_music');
                            end;
                            FEventsResult.events.Add(event);
                        end;
                    end;
                except
                    on E:exception do FLogFile.Log('GetEvents(3) Exception: '+e.message);
                end;
            end;
        end;
    end else begin
        result:=nil;
        FLogFile.Log('GetEvents Error: '+Fresult.GetErrorString);
    end;
end;

{ TSyncItemsResult }

constructor TSyncItemsResult.create;
begin
  items := TObjectlist.create;
  items.OwnsObjects:=true;
end;

destructor TSyncItemsResult.destroy;
begin
  items.Free;
end;

{ TGetEventsResult }

constructor TGetEventsResult.create;
begin
  events := TObjectlist.create;
  events.OwnsObjects:=true;

end;

destructor TGetEventsResult.destroy;
begin
  events.free;
end;


end.
